﻿package edu.uoc.tfc.glucodiari;


import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import org.json.JSONArray;
import org.json.JSONObject;

import java.sql.SQLException;



/**
 * Aquesta classe permet:
 * 1.- Recuperar i guardar els registres a la base de
 * dades SQLite, fent servir la classe SQLiteOpenHelper.
 * 2.- Crear un JSONArray amb tots els registres del diari.
 * 3.- Recuperar i calcular les dades necessàries per fer els informes
 *
 * NOTA: Per funcionar correctament les dates han de tenir el format YYYY-MM-DD
 *
 * @author: Xavier Asensio - (TFC App. Android - UOC).
 * @version: 1.0
 * @see <a href = "http://developer.android.com/guide/topics/data/data-storage.html#db" /> Storage Options </a>
 * @see <a href = "http://json.org" /> Introducing JSON </a>
 */

public class OperacionsDiari {

    private Context context = null;
    private OperacionsDiariOpenHelper diariOpenHelper = null;
    private SQLiteDatabase diariDB = null;
    /* Taula i camps */
    private static final String DATABASE_NAME = "GLUCODIARI.db";
    private static final int DATABASE_VERSION = 1;
    private static final String DATABASE_TABLE_NAME = "diari";
    public static final String DIARI_COLUMN_DATA = "data";
    public static final String DIARI_COLUMN_ESMORZAR = "esmorzar";
    public static final String DIARI_COLUMN_DINAR = "dinar";
    public static final String DIARI_COLUMN_SOPAR = "sopar";
    public static final String DIARI_COLUMN_NIT = "nit";
    /* SQL de creació de la taula */
    private static final String DATABASE_CREATION =
            "CREATE TABLE IF NOT EXISTS " + DATABASE_TABLE_NAME + " (" +
            DIARI_COLUMN_DATA + " DATE PRIMARY KEY, " +
            DIARI_COLUMN_ESMORZAR + " INTEGER, " +
            DIARI_COLUMN_DINAR + " INTEGER, " +
            DIARI_COLUMN_SOPAR + " INTEGER, " +
            DIARI_COLUMN_NIT + " INTEGER);";

    /**
     *  Constructor
     */
    public OperacionsDiari(Context ctx){
        this.context = ctx;
    }

    /**
     *  Classe privada per manipular la SQLite.
     */
    private static class OperacionsDiariOpenHelper extends SQLiteOpenHelper {
        public OperacionsDiariOpenHelper(Context context){
            super(context, DATABASE_NAME, null, DATABASE_VERSION);
        }

        @Override
        public void onCreate(SQLiteDatabase db){
            db.execSQL(DATABASE_CREATION);
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion){
            db.execSQL("DROP TABLE IF EXISTS " + DATABASE_TABLE_NAME);
            onCreate(db);
        }

    }

    /**
     * Mètode per obrir base de dades.
     * @return Objecte OperacionsDiari.
     */
    public OperacionsDiari open() throws SQLException {
        diariOpenHelper = new OperacionsDiariOpenHelper(context);
        diariDB = diariOpenHelper.getWritableDatabase();
        return this;
    }

    /**
     * Mètode per tancar la base de dades.
     */
    public void close(){
        diariOpenHelper.close();
    }

    /**
     * Mètode per guardar els valor corresponents al dia indicat.
     * !ATENCIO! El mètode assumeix que la data passada com a paràmetre és correcta i ben formada.
     * @param data Data de la qual es volen guardar els valors.
     * @param valors Array amb els valors de esmorzar, dinar, sopar i nit.
     * @return Retorna true si la inserció ha tingut èxit. false altrament.
     */
    public boolean InserirRegistre(String data, int[] valors) throws SQLException{
        ContentValues cv = new ContentValues();
        cv.put(DIARI_COLUMN_DATA, data);
        cv.put(DIARI_COLUMN_ESMORZAR, valors[0]);
        cv.put(DIARI_COLUMN_DINAR, valors[1]);
        cv.put(DIARI_COLUMN_SOPAR, valors[2]);
        cv.put(DIARI_COLUMN_NIT, valors[3]);
        /* Inserir el valor del dia seleccionat. Si existeix el registre, sobreescriure */
        long resultat = diariDB.insertWithOnConflict(DATABASE_TABLE_NAME, null, cv, SQLiteDatabase.CONFLICT_REPLACE);
        if (resultat == -1) {
            /* Error en inserció */
            return false;
        }else{
            /* Inserció OK */
            return true;
        }
    }

    /**
     * Mètode per recuperar els valor corresponents al dia indicat.
     * !ATENCIO! El mètode assumeix que la data passada com a paràmetre és correcta i ben formada.
     * @param data Data de la qual es volen recuperar els valors.
     * @return Array amb els valors d'esmorzar, dinar, sopar i nit.
     */
    public int[] RecuperarRegistre(String data) throws SQLException {
        /* Executar la consulta SQL per recuperar els valors */
        Cursor c = diariDB.rawQuery("SELECT * FROM " + DATABASE_TABLE_NAME + " WHERE " +
                DIARI_COLUMN_DATA + " ='" + data + "'", null);
        int[] resultats = new int[4];
        if (c.moveToFirst()) {
            /* S'ha trobat el registre, retornar les dades */
            resultats[0] = c.getInt(1);
            resultats[1] = c.getInt(2);
            resultats[2] = c.getInt(3);
            resultats[3] = c.getInt(4);
        }else{
            /* No s'ha trobat el registre, retorndar zeros */
            resultats[0] = 0;
            resultats[1] = 0;
            resultats[2] = 0;
            resultats[3] = 0;
        }
        c.close();
        return resultats;
    }

    /**
     * Mètode que obté dades dels valors entre dos dates donades.
     * !ATENCIO! El mètode assumeix que les dates passades com a paràmetre són correctes i ben formades.
     * @param dataInici Data inicial.
     * @param dataFi Data final .
     * @return Array amb els valors nombre registres, suma de valors, nombre per sobre, nombre per sota.
     */
    public int[] RegistresUltimsDies(String dataInici, String dataFi, int limitSuperior,
                                     int limitInferior) throws SQLException {
        /* Executar la consulta SQL per recuperar els valors */
        Cursor c = diariDB.rawQuery("SELECT * FROM " + DATABASE_TABLE_NAME + " WHERE " +
                DIARI_COLUMN_DATA + " BETWEEN '" + dataInici + "' AND '" + dataFi + "'", null);
        /* Calcular els resultats amb els registres obtinguts */
        int nombreRegistres = 0;
        int sumaValors = 0;
        int nombreRegistresSobreLimit = 0;
        int nombreRegistresSotaLimit = 0;
        c.moveToFirst();
        while (!c.isAfterLast()) {
            for (int i = 1; i < 5; i++) {
            /* Si el valor és diferent de zero el processem */
                if (c.getInt(i) != 0) {
                    nombreRegistres++;
                    if (c.getInt(i) > limitSuperior) {
                        /* Registre sobre el límit */
                        nombreRegistresSobreLimit++;
                    }
                    if (c.getInt(i) < limitInferior) {
                        /* Regsitre sota el límit */
                        nombreRegistresSotaLimit++;
                    }
                    /* Sumar el valor al total */
                    sumaValors = sumaValors + c.getInt(i);
                }
            }
            c.moveToNext();
        }
        int[] resultats = new int[4];
        resultats[0] = nombreRegistres;
        resultats[1] = sumaValors;
        resultats[2] = nombreRegistresSobreLimit;
        resultats[3] = nombreRegistresSotaLimit;
        c.close();
        return resultats;
    }

    /**
     * Crea un JSONArray amb tots els registres del diari.
     * @param usuari Codi d'usuari.
     * @return JSONArray amb el diari.
     */
    public JSONArray diariJSON(String usuari) throws Exception {
        /* Recuperar tots el registres del diari */
        Cursor cursor = diariDB.rawQuery("SELECT  * FROM " + DATABASE_TABLE_NAME, null );
        JSONArray resultSet = new JSONArray();
        cursor.moveToFirst();
        /* Afegir cada registre al JSONArray */
        while (!cursor.isAfterLast()) {
            JSONObject rowObject = new JSONObject();
            /* Carregar els valor en el JSONArray */
            rowObject.put("usuari", usuari);
            rowObject.put("data", cursor.getString(0));
            rowObject.put("esmorzar", cursor.getString(1));
            rowObject.put("dinar", cursor.getString(2));
            rowObject.put("sopar", cursor.getString(3));
            rowObject.put("nit", cursor.getString(4));
            resultSet.put(rowObject);
            cursor.moveToNext();
        }
        cursor.close();
        return resultSet;
    }

    /**
     * Mètode per recuperar els valor mig corresponents al dia indicat.
     * !ATENCIO! El mètode assumeix que la data passada com a paràmetre és correcta i ben formada.
     * @param data Data de la qual es vol calcular el valor mig.
     * @return int amb el valor mig.
     */
    public int ValorMig(String data) {
        int nombreValors = 0;
        int sumaValors = 0;
        /* Recuperar els valors */
        try {
            int[] resultats = RecuperarRegistre(data);
            for (int i = 0; i < 4; i++) {
            /* Si el valor és diferent de zero el processem */
                if (resultats[i] != 0) {
                    nombreValors++;
                    sumaValors = sumaValors + resultats[i];
                }
            }
        } catch (SQLException e) {
            return 0;
        }
        if (nombreValors == 0) {
            return 0;
        } else {
            return sumaValors / nombreValors;
        }
    }
}
